#include "mish.hpp"
#include <fstream>
#include <sstream>

using namespace std;


Graph::Graph(const char *instance_name) {
 
  char * buf, *tmp;
  string str;
  int idv=-1;
  int ide = -1;
  
  std::ifstream is;
  is.open(instance_name);
  
  if (!is.is_open()) {
    cerr<<"Instance file not found\n";
    exit(1);
  }
  
  while (!is.eof()) {

    getline(is,str); // process each line

    buf = new char [str.size()+1];
    strcpy (buf, str.c_str());

    // buf now contains a c-string copy of str
    //printf("%s\n", buf);
    fflush(stdout);
    switch (buf[0]) {
    case 'p': {
      tmp = strtok(buf, " "); // splits buf
      tmp = strtok(0, " "); // and considers the second substd::std::string
      tmp = strtok(0, " ");
      num_of_vertices = atoi(tmp);
      tmp = strtok(0, " "); // and for the second one
      num_of_edges = atoi(tmp);
      edges.clear();
      vertices.clear();
    }
      break;
    case 'v': {
      Vertex new_v;
      tmp = strtok(buf, " "); // splits buf among the spaces
      tmp = strtok(NULL, " "); // consider the first number
      if (marcoverbose == 3)
	cout<<"-"<<tmp<<" ";
      if (marcoverbose == 3)
	cout<<tmp<<" ";
      int v = atoi(tmp) - 1;
      assert(v>=0 && v<num_of_vertices);
      new_v.id=v;
      tmp = strtok(NULL, " "); // consider the next number
      int w = atoi(tmp);
      new_v.weight=w;
      idv++;
      vertices.insert(pair<const int, Vertex>(idv, new_v));	
    }
      break;
    case 'e': {
      // reads the edge information and add the edge between the 
      // corresponding nodes
      Edge edge;
      edge.members.clear();
      tmp = strtok(buf, " "); // splits buf among the spaces
      tmp = strtok(NULL, " "); // consider the first number
      edge.bound = atoi(tmp);
      tmp = strtok(NULL, " "); // consider the first number
      if (marcoverbose == 3)
	cout<<"-"<<tmp<<" ";
      while (tmp != NULL)
	{
	  if (marcoverbose == 3)
	    cout<<tmp<<" ";
	  int v = atoi(tmp) - 1;
	  assert(v>=0 && v<num_of_vertices);
	  edge.members.push_back(v);
	  tmp = strtok(NULL, " "); // consider the next number
	}
      ide++;
      edge.idx=ide;
      edges.insert(pair<const int, Edge>(ide, edge));	
    }
      break;
    case 'c':
    case ' ':
    case '\n':
      // ignore the comments and the empty lines
      break;
    default:
      //std::cerr << "Error while reading the input file " << id << std::endl
      //  << "the line " << buf << " was not understandable" << std::endl;
      break;
  delete[] buf;  
    }
    
  }
  
  is.close();
  
  cout<<"Instance read"<<endl;
  cout<<"vertices: "<<num_of_vertices<<endl;
  cout<<"edges: "<<num_of_edges<<endl;
}

void Graph::read_sln(const char *solfile_name) {

  char * buf, *tmp;
  string str;
  int idv=-1;
  int ide = -1; // for doublechecking file definition (counts the read edges)
  
  std::ifstream is;
  is.open(solfile_name);
  
  if (!is.is_open()) {
    cerr<<"Instance file not found\n";
    exit(1);
  }
  cost=0;
  while (!is.eof()) {

    getline(is,str); // process each line

    buf = new char [str.size()+1];
    strcpy (buf, str.c_str());

    fflush(stdout);
    switch (buf[0]) {
     case 's': {
      Vertex new_v;
      tmp = strtok(buf, " "); // splits buf among the spaces
      tmp = strtok(NULL, " "); // consider the first number
      if (marcoverbose == 3)
	cout<<"-"<<tmp<<" ";
      if (marcoverbose == 3)
	cout<<tmp<<" ";
      int v = atoi(tmp) - 1;
      assert(v>=0 && v<num_of_vertices);
      solution.push_back(v);
      cost+=vertices[v].weight;
    }
      break;
     case 'c':
    case ' ':
    case '\n':
      // ignore the comments and the empty lines
      break;
    default:
      //std::cerr << "Error while reading the input file " << id << std::endl
      //  << "the line " << buf << " was not understandable" << std::endl;
      break;
  delete[] buf;  
    }
    
  }
  
  is.close();
  
  cout<<"Solution read"<<endl;
  cout<<"vertices: "<<solution.size()<<endl;
  cout<<"cost: "<<cost<<endl;
}




void Graph::check_sln() {
  sort(solution.begin(),solution.end());
    
  bool is_stable_set=true;
  std::map<const int, Edge>::iterator pt;
  std::vector<int>::iterator pt2;
  std::vector<int>::iterator pt3;
  for (pt=edges.begin();pt!=edges.end();pt++)
    {
      sort(pt->second.members.begin(),pt->second.members.end());

      pt2=pt->second.members.begin();
      pt3=solution.begin();

      while ( pt2!=pt->second.members.end() )
	{
	  while (*pt3 < *pt2 && pt3!=solution.end())
	    pt3++;
	  
	  if (*pt3 > *pt2 || pt3==solution.end())
	    break;
	  pt2++;	  
	}
      if (pt2==pt->second.members.end())
	{
	  cerr<<"The solution is not an independent set"<<endl;
	  cerr<<"edge: (";
	  copy(pt->second.members.begin(),pt->second.members.end(),ostream_iterator<int>(cout, " "));
	  cerr<<") is fully included in ";
	  copy(solution.begin(),solution.end(),ostream_iterator<int>(cerr, " "));
	  cerr<<endl;
	  is_stable_set=false;
	}
    }
  if (is_stable_set)
    cout<<"\nThe solution has cost "<<cost<<" and is a stable set."<<endl<<endl;
  else
    cout<<"\nThe solution is NOT a stable set."<<endl<<endl;
}

